<?php

namespace Bixo\Database\Http\Controllers;

use Exception;
use Litepie\Http\Controllers\ResourceController as BaseController;
use Bixo\Database\Forms\Call as CallForm;
use Bixo\Database\Http\Requests\CallResourceRequest;
use Bixo\Database\Http\Resources\CallResource;
use Bixo\Database\Http\Resources\CallsCollection;
use Bixo\Database\Models\Call;
use Bixo\Database\Actions\CallAction;
use Bixo\Database\Actions\CallActions;


/**
 * Resource controller class for call.
 */
class CallResourceController extends BaseController
{

    /**
     * Initialize call resource controller.
     *
     *
     * @return null
     */
    public function __construct()
    {
        parent::__construct();
        $this->middleware(function ($request, $next) {
            $this->form = CallForm::grouped(false)
                            ->setAttributes()
                            ->toArray();
            $this->modules = $this->modules(config('bixo.database.modules'), 'database', guard_url('database'));
            return $next($request);
        });
    }

    /**
     * Display a list of call.
     *
     * @return Response
     */
    public function index(CallResourceRequest $request)
    {
        $request = $request->all();
        $page = CallActions::run('paginate', $request);

        $data = new CallsCollection($page);

        $form = $this->form;
        $modules = $this->modules;

        return $this->response->setMetaTitle(trans('database::call.names'))
            ->view('database::call.index')
            ->data(compact('data', 'modules', 'form'))
            ->output();

    }

    /**
     * Display call.
     *
     * @param Request $request
     * @param Model   $call
     *
     * @return Response
     */
    public function show(CallResourceRequest $request, Call $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new CallResource($model);
        return $this->response
            ->setMetaTitle(trans('app.view') . ' ' . trans('database::call.name'))
            ->data(compact('data', 'form', 'modules'))
            ->view('database::call.show')
            ->output();
    }

    /**
     * Show the form for creating a new call.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(CallResourceRequest $request, Call $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new CallResource($model);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('database::call.name'))
            ->view('database::call.create')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Create new call.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(CallResourceRequest $request, Call $model)
    {
        try {
            $request = $request->all();
            $model = CallAction::run('store', $model, $request);
            $data = new CallResource($model);
            return $this->response->message(trans('messages.success.created', ['Module' => trans('database::call.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('database/call/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/database/call'))
                ->redirect();
        }

    }

    /**
     * Show call for editing.
     *
     * @param Request $request
     * @param Model   $call
     *
     * @return Response
     */
    public function edit(CallResourceRequest $request, Call $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new CallResource($model);
        // return view('database::call.edit', compact('data', 'form', 'modules'));

        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('database::call.name'))
            ->view('database::call.edit')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Update the call.
     *
     * @param Request $request
     * @param Model   $call
     *
     * @return Response
     */
    public function update(CallResourceRequest $request, Call $model)
    {
        try {
            $request = $request->all();
            $model = CallAction::run('update', $model, $request);
            $data = new CallResource($model);

            return $this->response->message(trans('messages.success.updated', ['Module' => trans('database::call.name')]))
                ->code(204)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('database/call/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('database/call/' .  $model->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the call.
     *
     * @param Model   $call
     *
     * @return Response
     */
    public function destroy(CallResourceRequest $request, Call $model)
    {
        try {

            $request = $request->all();
            $model = CallAction::run('destroy', $model, $request);
            $data = new CallResource($model);

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('database::call.name')]))
                ->code(202)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('database/call/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('database/call/' .  $model->getRouteKey()))
                ->redirect();
        }

    }
}
