<?php

namespace Bixo\Database\Http\Controllers;

use Exception;
use Litepie\Http\Controllers\ResourceController as BaseController;
use Bixo\Database\Forms\Owner as OwnerForm;
use Bixo\Database\Http\Requests\OwnerResourceRequest;
use Bixo\Database\Http\Resources\OwnerResource;
use Bixo\Database\Http\Resources\OwnersCollection;
use Bixo\Database\Models\Owner;
use Bixo\Database\Actions\OwnerAction;
use Bixo\Database\Actions\OwnerActions;


/**
 * Resource controller class for owner.
 */
class OwnerResourceController extends BaseController
{

    /**
     * Initialize owner resource controller.
     *
     *
     * @return null
     */
    public function __construct()
    {
        parent::__construct();
        $this->middleware(function ($request, $next) {
            $this->form = OwnerForm::grouped(false)
                            ->setAttributes()
                            ->toArray();
            $this->modules = $this->modules(config('bixo.database.modules'), 'database', guard_url('database'));
            return $next($request);
        });
    }

    /**
     * Display a list of owner.
     *
     * @return Response
     */
    public function index(OwnerResourceRequest $request)
    {
        $request = $request->all();
        $page = OwnerActions::run('paginate', $request);

        $data = new OwnersCollection($page);

        $form = $this->form;
        $modules = $this->modules;

        return $this->response->setMetaTitle(trans('database::owner.names'))
            ->view('database::owner.index')
            ->data(compact('data', 'modules', 'form'))
            ->output();

    }

    /**
     * Display owner.
     *
     * @param Request $request
     * @param Model   $owner
     *
     * @return Response
     */
    public function show(OwnerResourceRequest $request, Owner $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new OwnerResource($model);
        return $this->response
            ->setMetaTitle(trans('app.view') . ' ' . trans('database::owner.name'))
            ->data(compact('data', 'form', 'modules'))
            ->view('database::owner.show')
            ->output();
    }

    /**
     * Show the form for creating a new owner.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(OwnerResourceRequest $request, Owner $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new OwnerResource($model);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('database::owner.name'))
            ->view('database::owner.create')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Create new owner.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(OwnerResourceRequest $request, Owner $model)
    {
        try {
            $request = $request->all();
            $model = OwnerAction::run('store', $model, $request);
            $data = new OwnerResource($model);
            return $this->response->message(trans('messages.success.created', ['Module' => trans('database::owner.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('database/owner/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/database/owner'))
                ->redirect();
        }

    }

    /**
     * Show owner for editing.
     *
     * @param Request $request
     * @param Model   $owner
     *
     * @return Response
     */
    public function edit(OwnerResourceRequest $request, Owner $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new OwnerResource($model);
        // return view('database::owner.edit', compact('data', 'form', 'modules'));

        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('database::owner.name'))
            ->view('database::owner.edit')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Update the owner.
     *
     * @param Request $request
     * @param Model   $owner
     *
     * @return Response
     */
    public function update(OwnerResourceRequest $request, Owner $model)
    {
        try {
            $request = $request->all();
            $model = OwnerAction::run('update', $model, $request);
            $data = new OwnerResource($model);

            return $this->response->message(trans('messages.success.updated', ['Module' => trans('database::owner.name')]))
                ->code(204)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('database/owner/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('database/owner/' .  $model->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the owner.
     *
     * @param Model   $owner
     *
     * @return Response
     */
    public function destroy(OwnerResourceRequest $request, Owner $model)
    {
        try {

            $request = $request->all();
            $model = OwnerAction::run('destroy', $model, $request);
            $data = new OwnerResource($model);

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('database::owner.name')]))
                ->code(202)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('database/owner/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('database/owner/' .  $model->getRouteKey()))
                ->redirect();
        }

    }
}
