<?php

namespace Bixo\Database\Http\Controllers;

use Exception;
use Litepie\Http\Controllers\ResourceController as BaseController;
use Bixo\Database\Forms\Upload as UploadForm;
use Bixo\Database\Http\Requests\UploadResourceRequest;
use Bixo\Database\Http\Resources\UploadResource;
use Bixo\Database\Http\Resources\UploadsCollection;
use Bixo\Database\Models\Upload;
use Bixo\Database\Actions\UploadAction;
use Bixo\Database\Actions\UploadActions;


/**
 * Resource controller class for upload.
 */
class UploadResourceController extends BaseController
{

    /**
     * Initialize upload resource controller.
     *
     *
     * @return null
     */
    public function __construct()
    {
        parent::__construct();
        $this->middleware(function ($request, $next) {
            $this->form = UploadForm::grouped(false)
                            ->setAttributes()
                            ->toArray();
            $this->modules = $this->modules(config('bixo.database.modules'), 'database', guard_url('database'));
            return $next($request);
        });
    }

    /**
     * Display a list of upload.
     *
     * @return Response
     */
    public function index(UploadResourceRequest $request)
    {
        $request = $request->all();
        $page = UploadActions::run('paginate', $request);

        $data = new UploadsCollection($page);

        $form = $this->form;
        $modules = $this->modules;

        return $this->response->setMetaTitle(trans('database::upload.names'))
            ->view('database::upload.index')
            ->data(compact('data', 'modules', 'form'))
            ->output();

    }

    /**
     * Display upload.
     *
     * @param Request $request
     * @param Model   $upload
     *
     * @return Response
     */
    public function show(UploadResourceRequest $request, Upload $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new UploadResource($model);
        return $this->response
            ->setMetaTitle(trans('app.view') . ' ' . trans('database::upload.name'))
            ->data(compact('data', 'form', 'modules'))
            ->view('database::upload.show')
            ->output();
    }

    /**
     * Show the form for creating a new upload.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(UploadResourceRequest $request, Upload $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new UploadResource($model);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('database::upload.name'))
            ->view('database::upload.create')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Create new upload.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(UploadResourceRequest $request, Upload $model)
    {
        try {
            $request = $request->all();
            $model = UploadAction::run('store', $model, $request);
            $data = new UploadResource($model);
            return $this->response->message(trans('messages.success.created', ['Module' => trans('database::upload.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('database/upload/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/database/upload'))
                ->redirect();
        }

    }

    /**
     * Show upload for editing.
     *
     * @param Request $request
     * @param Model   $upload
     *
     * @return Response
     */
    public function edit(UploadResourceRequest $request, Upload $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new UploadResource($model);
        // return view('database::upload.edit', compact('data', 'form', 'modules'));

        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('database::upload.name'))
            ->view('database::upload.edit')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Update the upload.
     *
     * @param Request $request
     * @param Model   $upload
     *
     * @return Response
     */
    public function update(UploadResourceRequest $request, Upload $model)
    {
        try {
            $request = $request->all();
            $model = UploadAction::run('update', $model, $request);
            $data = new UploadResource($model);

            return $this->response->message(trans('messages.success.updated', ['Module' => trans('database::upload.name')]))
                ->code(204)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('database/upload/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('database/upload/' .  $model->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the upload.
     *
     * @param Model   $upload
     *
     * @return Response
     */
    public function destroy(UploadResourceRequest $request, Upload $model)
    {
        try {

            $request = $request->all();
            $model = UploadAction::run('destroy', $model, $request);
            $data = new UploadResource($model);

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('database::upload.name')]))
                ->code(202)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('database/upload/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('database/upload/' .  $model->getRouteKey()))
                ->redirect();
        }

    }
}
